
fixture_datasource_dictionary_ma001 = r"""
{
"settlement_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\gbr_stle_stle_pt_s0_naturalearth_pp.shp"
"airports_points": "D:\MapAction\2019-06-12-GBR\GIS\2_Active_Data\232_tran\scr_tran_air_pt_s1_ourairports_pp.shp"
}
"""


fixture_layer_description_ma001 = r"""
-
   m: Main Map
   layer-group: None
   layer-name: Settlement - Places - pt
   source-folder: 229_stle
   regex: ^ XXX_stle_stl_pt_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: "SettleType" IN('national_capital', 'city')
   visable: Yes
-
   m: Main Map
   layer-group: Transport - Points
   layer-name: Transport - Airports - pt
   source-folder: 232_tran
   regex: ^ XXX_trans_air_pt_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Transport - Points
   layer-name: Transport - Seaports - pt
   source-folder: 232_tran
   regex: ^ XXX_trans_por_pt_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Lines
   layer-name: Elevation - Coastline - ln
   source-folder: 211_elev
   regex: ^ XXX_elev_cst_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: none
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Lines
   layer-name: Borders - Admin1 - ln
   source-folder: 202_admn
   regex: ^ XXX_admn_ad1_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Lines
   layer-name: Borders - Admin2 - ln
   source-folder: 202_admn
   regex: ^ XXX_admn_ad2_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Transport - Lines
   layer-name: Transport - Rail - ln
   source-folder: 232_tran
   regex: ^ XXX_tran_rrd_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Transport - Lines
   layer-name: Transport - Road - ln
   source-folder: 232_tran
   regex: ^ XXX_tran_rds_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: None
   layer-name: Cartography - Feather - pt
   source-folder: 207_carto
   regex: ^ (?!(XXX))_carto_fea_py_s0_mapaction_pp(_(.+)
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Physical
   layer-name: Physical - Waterbody - py
   source-folder: 221_phys
   regex: tbd
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Physical
   layer-name: Physical - River - ln
   source-folder: 221_phys
   regex: ^ XXX_phys_riv_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Polygons
   layer-name: Admin - Admin2 - py
   source-folder: 202_admn
   regex: ^ XXX_admn_ad2_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Polygons
   layer-name: Admin - Admin1 - py
   source-folder: 202_admn
   regex: ^ XXX_admn_ad1_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Polygons
   layer-name: Admin - AffectedCountry - py
   source-folder: 202_admn
   regex: ^ XXX_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Admin - Polygons
   layer-name: Admin - SurroundingCountry - py
   source-folder: 202_admn
   regex: ^ (?!(XXX))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: ADM0_NAME <> '[reference country]'
   visable: Yes
-
   m: Main Map
   layer-group: Elevation
   layer-name: Physical - Sea - py
   source-folder: 221_phys
   regex: ^ XXX_phys_ocn_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Elevation
   layer-name: Elevation - DEM - ras
   source-folder: 211_elev
   regex: ^ XXX_elev_dem_ras_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Elevation
   layer-name: Elevation - Hillshade - ras
   source-folder: 211_elev
   regex: ^ XXX_elev_hsh_ras_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Elevation
   layer-name: Elevation - Curvature - ras
   source-folder: 211_elev
   regex: tbd
   query-definition: None
   visable: Yes
-
   m: Main Map
   layer-group: Legend
   layer-name: Legend - Road - ln
   source-folder: 232_tran
   regex: ^ XXX_tran_rds_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: No
-
   m: Main Map
   layer-group: Legend
   layer-name: Legend - WaterBody - py
   source-folder: 221_phys
   regex: tbd
   query-definition: None
   visable: No
-
   m: Main Map
   layer-group: Legend
   layer-name: Elevation - Elevation - ras
   source-folder: 211_elev
   regex: ^ XXX_elev_dem_ras_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: No
-
   m: Location Map
   layer-group: None
   layer-name: Location - Coastline - ln
   source-folder: 211_elev
   regex: ^ XXX_elev_cst_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Location Map
   layer-group: None
   layer-name: Location - Admin1 - ln
   source-folder: 202_admn
   regex: ^ XXX_admn_ad1_ln_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Location Map
   layer-group: None
   layer-name: Location - AffectedCountry - py
   source-folder: 202_admn
   regex: ^ XXX_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: None
   visable: Yes
-
   m: Location Map
   layer-group: None
   layer-name: Location - SurroundingCountry - py
   source-folder: 202_admn
   regex: ^ (?!(XXX))_admn_ad0_py_(.*?)_(.*?)_([phm][phm])(_(.+))
   query-definition: "ADM0_NAME" <> '[reference country]'
   visable: Yes
"""



import arcpy
import argparse
import json
import errno
import glob
import os
from shutil import copyfile
from slugify import slugify
from PIL import Image
from zipfile import ZipFile
from resizeimage import resizeimage
from map_chef import MapChef
from mapactionpy_controller.map_cookbook import MapCookbook
from mapactionpy_controller.data_source import DataSource
from mapactionpy_controller.layer_properties import LayerProperties
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
from mapactionpy_controller.event import Event
from mapactionpy_controller.xml_exporter import XmlExporter


class ArcMapRunner:
    """
    ArcMapRunner - Executes the ArcMap automation methods
    """

    # TODO: asmith 2020/03/03
    #
    # 1) I think that this constructor could be made much simpler by only requiring either
    # an Event object or an event_description.json file. Nearly all of the other parameters
    # could be detrived from oue of those.
    #
    # 2) If the path to the event_description.json file is used, make it the path to the file
    # itself and not the containing directory. That would avoid the need to hardcode the filename
    # of the event_description.json file.
    #
    # 4) I do not see the need for the ArcMapRunner to have it's own members which duplicate those
    # of the Event object or the CMF object. Eg. `myrunner.cmf.layer_properties` is fine.
    #
    # 5) If there is a case for allowing the user to manaually specify or override all of the values
    # that are typically included in the cmf_description.json file and/or the event_description.json
    # file than the wrangling and validation of those commandline args should be done outside of this
    # class and in the mapactionpy_controller module.
    def __init__(self,
                 templateFile,
                 eventConfig,
                 productName):
        self.mxdTemplate = templateFile
        self.cookbookFile = None
        self.layerPropertiesFile = None
        self.productName = productName
        self.event = eventConfig
        self.replaceOnly = False

        # TODO: asmith 2020/03/03
        # Do not hard code "real" values. Setting to `None` would seem safer here as that
        # ensures that if the versionNumber or mapNumber are not set correctly later in the
        # code, these default values do not accidently result in real outputs being overwritten
        self.versionNumber = 1
        self.mapNumber = "MA001"
        self.exportMap = False
        self.minx = 0
        self.miny = 0
        self.maxx = 0
        self.maxy = 0
        self.chef = None
        self.crashMoveFolder = CrashMoveFolder(self.event.cmf_descriptor_path)

        # TODO: asmith 2020/03/03
        # The name `self.layerDefinition` is unclear and should be changed.
        #   * As far as I can see this object encapsulates the properties of multiple layers
        #     (not merely multiple properties of a single layer).
        #   * Also `layerDefinition` is easily confussed with the DefinitionQuery of a layer (which
        #     it's not).
        self.layerDefinition = LayerProperties(self.crashMoveFolder, '.lyr')

        # TODO: asmith 2020/03/03
        # Much of here to line 132 is unrequired, as the CrashMoveFolder class already has a method
        # for verifying the existence of all of the relevant files and directories. By default this
        # happens when a CrashMoveFolder object is created. For the paranoid these two lines:
        # ```
        # if not self.cmf.verify_paths():
        #    raise ValueError("Cannot find paths and directories referenced by cmf {}".format(self.cmf.path))
        # ```

        # The following checks are used if the CMF description is over-ridden by the command line parameters
        if self.cookbookFile is None:
            if self.crashMoveFolder is not None:
                self.cookbookFile = self.crashMoveFolder.map_definitions
            else:
                raise Exception("Error: Could not derive cookbook file from " + self.event.cmf_descriptor_path)

        self.cookbook = MapCookbook(self.crashMoveFolder, self.layerDefinition)
        # self.cookbook = MapCookbook(self.cookbookFile)

        try:
            self.recipe = self.cookbook.products[productName]
        except KeyError:
            raise Exception("Error: Could not find recipe for product: \"" + productName + "\" in " + self.cookbookFile)

        if self.layerPropertiesFile is None:
            if self.crashMoveFolder is not None:
                self.layerPropertiesFile = self.crashMoveFolder.layer_properties
            else:
                raise Exception("Error: Could not derive layer config file from " + self.event.cmf_descriptor_path)
        # OLD 132
        # TODO: asmith 2020/03/03
        # The name `self.layerDefinition` is unclear and should be changed.
        #   * As far as I can see this object encapsulates the properties of multiple layers
        #     (not merely multiple properties of a single layer).
        #   * Also `layerDefinition` is easily confused with the DefinitionQuery of a layer (which
        #     it's not).
        self.layerDefinition = LayerProperties(self.crashMoveFolder, '.lyr')

    # TODO: asmith 2020/03/03
    # method name is unclear. Generate what? How does this relate to
    # the `generateXml` method?

    def generate(self):
        # Construct a Crash Move Folder object if the cmf_description.json exists
        generationRequired = False
        if self.mxdTemplate is None:
            self.mxdTemplate, self.mapNumber, self.versionNumber, generationRequired = self.get_template(
                self.cookbookFile, self.crashMoveFolder, self.productName)
        else:
            generationRequired = True
        if (generationRequired):
            mxd = arcpy.mapping.MapDocument(self.mxdTemplate)

            self.chef = MapChef(mxd,
                                self.cookbook,
                                self.layerDefinition,
                                self.crashMoveFolder,
                                self.event,
                                self.versionNumber)
            self.chef.cook(self.recipe, self.replaceOnly)
            self.chef.alignLegend(self.event.orientation)

            # Output the Map Generation report alongside the MXD
            reportJsonFile = self.mxdTemplate.replace(".mxd", ".json")
            with open(reportJsonFile, 'w') as outfile:
                outfile.write(self.chef.report_as_json())
        return generationRequired

    # TODO: asmith 2020/03/03
    # There is a lot going on in this method and I'm not sure I understand all of it. However here
    # are some thoughts:
    #
    # 1) If the MapCookbook class had a method that validated the existance of all of the
    # templates it refers to then this method could be simplied somewhat. eg `self.recipe.template_mxd`
    # This would also make it simpler for the Cookbook to be tested/validated earlier and without
    # requiring any data; See https://wiki.mapaction.org/display/orgdev/Automatable+tests+of+CMF+contents
    #
    # 2) Why is the arcGisVersion required? Is this just to navigate the naming convention for mxd templates?
    # Would it be appropriate and/or simpler to ammend the name of the template files? They are already in a
    # dir called 'arcgis_10_6'. Or is there another arcpy related reason why it is required? Without it, it
    # would be possible to move this whole method over to the controller where it could be re-used for the
    # QGIS implementation.
    #
    # 3) I think it would be more consistent to use the naming convention classes for the mxd template; eg
    # https://github.com/mapaction/mapactionpy_controller/pull/38 This helps avoid the need to hardcode the
    # naming convention for input mxd templates.
    #
    # 4) Is it possible to aviod the need to hardcode the naming convention for the output mxds? Eg could a
    # String.Template be specified within the Cookbook?
    # https://docs.python.org/2/library/string.html#formatspec
    # https://www.python.org/dev/peps/pep-3101/
    def get_template(self, cookbookFile, crashMoveFolder, productName):
        arcGisVersion = crashMoveFolder.arcgis_version

        # Need to get the theme from the recipe to get the path to the MXD

        mapNumberTemplateFileName = self.recipe.mapnumber + "_" + self.event.orientation + ".mxd"
        mapNumberTemplateFilePath = os.path.join(crashMoveFolder.map_templates, mapNumberTemplateFileName)
        if os.path.exists(mapNumberTemplateFilePath):
            srcTemplateFile = mapNumberTemplateFilePath
            # In this instance, we only want to replace the datasource, everything else should say as is
            self.replaceOnly = True
        else:
            self.exportMap = self.recipe.export
            # TODO: asmith 2020/03/03
            # This will fail is the category is specified in the recipe in a different case to the
            # the case used in for the template filename.
            if (self.recipe.category.lower() == "reference"):
                templateFileName = arcGisVersion + "_" + \
                    self.recipe.category + \
                    "_" + \
                    self.event.orientation + \
                    "_bottom.mxd"
            elif (self.recipe.category.lower() == "ddp reference"):
                templateFileName = arcGisVersion + "_ddp_reference_" + self.event.orientation + ".mxd"
            elif (self.recipe.category.lower() == "thematic"):
                templateFileName = arcGisVersion + "_" + self.recipe.category + "_" + self.event.orientation + ".mxd"
            else:
                raise Exception("Error: Could not get source MXD from: " + crashMoveFolder.map_templates)

            srcTemplateFile = os.path.join(crashMoveFolder.map_templates, templateFileName)

        mapNumberDirectory = os.path.join(crashMoveFolder.map_projects, self.recipe.mapnumber)

        if not(os.path.isdir(mapNumberDirectory)):
            os.mkdir(mapNumberDirectory)

        # Construct MXD name
        mapFileName = slugify(unicode(productName))
        versionNumber = self.get_next_map_version_number(mapNumberDirectory, self.recipe.mapnumber, mapFileName)
        previousReportFile = self.recipe.mapnumber + "-v" + str((versionNumber-1)).zfill(2) + "_" + mapFileName + ".json"  # noqa
        copiedFile = "NOT SET"
        generationRequired = True
        if (os.path.exists(os.path.join(mapNumberDirectory, previousReportFile))):
            generationRequired = self.haveDataSourcesChanged(os.path.join(mapNumberDirectory, previousReportFile))

        if (generationRequired is True):
            mapFileName = self.recipe.mapnumber + "-v" + str(versionNumber).zfill(2) + "_" + mapFileName + ".mxd"
            copiedFile = os.path.join(mapNumberDirectory, mapFileName)
            copyfile(srcTemplateFile, copiedFile)

        return copiedFile, self.recipe.mapnumber, versionNumber, generationRequired

    # TODO: asmith 2020/03/03
    # Instinctively I would like to see this moved to the MapReport class with an __eq__ method which
    # would look very much like this one.
    def haveDataSourcesChanged(self, previousReportFile):
        returnValue = False
        with open(previousReportFile, 'r') as myfile:
            data = myfile.read()
            # parse file
            obj = json.loads(data)
            for result in obj['results']:
                dataFile = os.path.join(self.event.path, (result['dataSource'].strip('/')))
                previousHash = result.get('hash', "")
                ds = DataSource(dataFile)
                latestHash = ds.calculate_checksum()
                if (latestHash != previousHash):
                    returnValue = True
                    break
        return returnValue

    # TODO: asmith 2020/03/03
    #
    # 1) Please avoid hardcoding the naming convention for the mxds wherever possible. The Naming Convention
    # classes can avoid the need to hardcode the naming convention for the input mxd templates. It might be
    # possible to avoid the need to hardcode the naming convention for the output mxds using a
    # String.Template be specified within the Cookbook?
    # https://docs.python.org/2/library/string.html#formatspec
    # https://www.python.org/dev/peps/pep-3101/
    #
    # 2) This only checks the filename for the mxd - it doesn't check the values within the text element of
    # the map layout view (and hence the output metadata).
    def get_next_map_version_number(self, mapNumberDirectory, mapNumber, mapFileName):
        versionNumber = 0
        files = glob.glob(mapNumberDirectory + os.sep + mapNumber+'-v[0-9][0-9]_' + mapFileName + '.mxd')
        for file in files:
            versionNumber = int(os.path.basename(file).replace(mapNumber + '-v', '').replace(('_' + mapFileName+'.mxd'), ''))  # noqa
        versionNumber = versionNumber + 1
        if (versionNumber > 99):
            versionNumber = 1
        return versionNumber

    """
    Generates all file for export
    """
    # TODO: asmith 2020/03/03
    # This method is very long. Separate into multiple method. Some fo these will can be entirely
    # functional with no side-effects (eg wrangling parameters) whilst other will be highly proceedural
    # where the actual files writen to the filesystem.
    #
    # As a suggestion; it could be broken up something like this:
    #
    # export(self)
    #   """
    #   Accumulate parameters for export XML, then calls _do_export(....)
    #   """
    #
    #
    # _do_export(self, lots, of, specific, args)
    #   """
    #   Does the exporting of the PDF, Jpeg and thumbnail files.
    #   """
    #
    #
    # _process_query_column_name(...)
    #   """
    #   Accumulate parameters for export XML, then calls _do_export(....)
    #   """
    #
    # _zip_exported_files(....)
    #   """
    #   Accumulate parameters for export XML, then calls _do_export(....)
    #   """
    #
    # Other comments have been added thoughout the method.

    def export(self):
        # TODO: asmith 2020/03/03
        # 1) Separate the section "Accumulate parameters for export XML" into it's own method
        # 2) Please avoid hardcoding the naming convention for the output mxds.

        # Accumulate parameters for export XML
        exportParams = {}
        versionString = "v" + str(self.versionNumber).zfill(2)
        exportDirectory = os.path.join(self.crashMoveFolder.export_dir,
                                       self.mapNumber,
                                       versionString).replace('/', '\\')
        exportParams["exportDirectory"] = exportDirectory
        try:
            os.makedirs(exportDirectory)
        except OSError as exc:  # Python >2.5
            # Note 'errno.EEXIST' is not a typo. There should be two 'E's.
            # https://docs.python.org/2/library/errno.html#errno.EEXIST
            if exc.errno == errno.EEXIST and os.path.isdir(exportDirectory):
                pass
            else:
                raise

        # TODO: asmith 2020/03/03
        # End of method for the section "Accumulate parameters for export XML"

        # TODO: asmith 2020/03/03
        # Separate this section into a method named something like
        # _do_export(self, lots, of, specific, args)
        mxd = arcpy.mapping.MapDocument(self.mxdTemplate)

        coreFileName = os.path.splitext(os.path.basename(self.mxdTemplate))[0]
        exportParams["coreFileName"] = coreFileName
        productType = "mapsheet"
        exportParams["productType"] = productType

        pdfFileLocation = self.exportPdf(coreFileName, exportDirectory, mxd, exportParams)
        jpgFileLocation = self.exportJpeg(coreFileName, exportDirectory, mxd, exportParams)
        pngThumbNailFileLocation = self.exportPngThumbNail(coreFileName, exportDirectory, mxd, exportParams)

        # TODO: asmith 2020/03/03
        # End of method named something like
        # _do_export(self, lots, of, specific, args)

        #######################################################################################################
        # TODO: asmith 2020/03/03
        # Seperate this section into a method nameded something like
        # _process_query_column_name(...)
        if self.recipe.atlas:
            # TODO: asmith 2020/03/03
            #
            # 1) Please do not hard code layer file names! If a particular layer needs to have a special
            # meaning then this should be explict in the structure of the mapCookBook.json and/or
            # layerProperties.json files.
            #
            # 2) It looks like this is a workaround for the fact that this isn't an easy way to programatically
            # identify specific mapFrames without hardcoding the name of the mapFrame. Is there anywhere a
            # datamodel or class which encapsulates all of the elemnts which must be present within an mxd
            # in order for the automation tools to work? (eg:
            #     * title Text Element
            #     * description Text Element
            #     * datasources Text Element
            #     * map_map Map Frame
            #     * location_map Map Frame
            #     * legend Legend Element
            #     * etc.....
            #
            # It ought to be possible to have a method somewhere within the module like this:
            # ```
            #     is_suitable_for_automation(mxd)  # Returns Boolean
            # ```

            # Disable view of Affected Country
            locationMapLayerName = "locationmap-admn-ad0-py-s0-locationmaps"  # Hard-coded
            layerDefinition = self.layerDefinition.properties.get(locationMapLayerName)
            locationMapDataFrameName = layerDefinition.mapFrame
            locationMapDataFrame = arcpy.mapping.ListDataFrames(mxd, locationMapDataFrameName)[0]
            locationMapLyr = arcpy.mapping.ListLayers(mxd, locationMapLayerName, locationMapDataFrame)[0]
            locationMapDataFrame.extent = locationMapLyr.getExtent()
            locationMapLyr.visible = False

            # TODO: asmith 2020/03/03
            # Why is it necessary to have two loops through the layers in a recipe? eg
            # ```
            #     for layer in self.recipe.layers:
            # ```
            # There is this one here and a seperate one within the `cook` method of the MapChef class.
            # In the MapChef.cook() method the mxd is altered and then saved afterwards. It looks like
            # the mxd can be altered in ArcMapRunner.export() [specificly the zoom extent altered] but
            # then not saved afterwards. Is this intentional?
            for layer in self.recipe.layers:
                # TODO: asmith 2020/03/03
                #
                # 1) The snip-it `if (layer.get('columnName', None) is not None):` occurs in a couple of
                # locations (here and in MapRecipe.containsQueryColumn()). As a double negitive it is not
                # clear what is meant (I cannot find a `columnName` entry in the example mapCookbook.json).
                # Would it help to expand the object model for the `layer` class to encapsulate this?
                if (layer.get('columnName', None) is not None):
                    layerName = layer.get('name')
                    queryColumn = layer.get('columnName')
                    fieldNames = [queryColumn]
                    # For each layer and column name, export a regional map
                    layerDefinition = self.layerDefinition.properties.get(layerName)
                    df = arcpy.mapping.ListDataFrames(mxd, layerDefinition.mapFrame)[0]
                    lyr = arcpy.mapping.ListLayers(mxd, layerName, df)[0]

                    # TODO: asmith 2020/03/03
                    #
                    # Presumably `regions` here means admin1 boundaries or some other internal
                    # administrative devision?
                    regions = list()
                    with arcpy.da.UpdateCursor(lyr.dataSource, fieldNames) as cursor:
                        for row in cursor:
                            regions.append(row[0])

                    # TODO: asmith 2020/03/03
                    # This loop appear to simulate the behaviour of Data Driven Pages - is that right?
                    # Simulating Data Driven Pages to fine given the limitations in the arcpy API for
                    # controlling them.
                    #
                    # 1) If this is DDP, then I presume that this is triggered by the
                    # `if (layer.get('columnName', None) is not None):` line above? If so then please
                    # could we change the format of the mapCookBook.json so that this is more apparent
                    # to the reader of the mapCookBook.json file?
                    #
                    # 2) There appear to be a lot of hardcoded elements (eg the title and the map number)
                    # Is it possible to eliminate (or at least minimse these)?
                    for region in regions:
                        # TODO: asmith 2020/03/03
                        # Please do not hardcode mapFrame names. If a particular mapframe as a special
                        # meaning then this should be explicit in the structure of the mapCookBook.json
                        # and/or layerProperties.json files.
                        dataFrameName = "Main map"
                        df = arcpy.mapping.ListDataFrames(mxd, dataFrameName)[0]

                        # Select the next region
                        query = "\"" + queryColumn + "\" = \'" + region + "\'"
                        arcpy.SelectLayerByAttribute_management(lyr, "NEW_SELECTION", query)

                        # Get the extent of the selected area
                        df.extent = lyr.getSelectedExtent()

                        # Create a polygon using the bounding box
                        array = arcpy.Array()
                        array.add(df.extent.lowerLeft)
                        array.add(df.extent.lowerRight)
                        array.add(df.extent.upperRight)
                        array.add(df.extent.upperLeft)
                        # ensure the polygon is closed
                        array.add(df.extent.lowerLeft)
                        # Create the polygon object
                        polygon = arcpy.Polygon(array, df.extent.spatialReference)

                        array.removeAll()

                        # Export the extent to a shapefile
                        shapeFileName = "extent_" + slugify(unicode(region)).replace('-', '')
                        shpFile = shapeFileName + ".shp"

                        if arcpy.Exists(os.path.join(exportDirectory, shpFile)):
                            arcpy.Delete_management(os.path.join(exportDirectory, shpFile))
                        arcpy.CopyFeatures_management(polygon, os.path.join(exportDirectory, shpFile))

                        # For the 'extent' layer...
                        locationMapDataFrameName = "Location map"
                        locationMapDataFrame = arcpy.mapping.ListDataFrames(mxd, locationMapDataFrameName)[0]
                        extentLayerName = "locationmap-s0-py-extent"
                        extentLayer = arcpy.mapping.ListLayers(mxd, extentLayerName, locationMapDataFrame)[0]

                        # Update the layer
                        extentLayer.replaceDataSource(exportDirectory, 'SHAPEFILE_WORKSPACE', shapeFileName)
                        arcpy.RefreshActiveView()

                        # In Main map, zoom to the selected region
                        dataFrameName = "Main map"
                        df = arcpy.mapping.ListDataFrames(mxd, dataFrameName)[0]
                        arcpy.SelectLayerByAttribute_management(lyr, "NEW_SELECTION", query)
                        df.extent = lyr.getSelectedExtent()

                        for elm in arcpy.mapping.ListLayoutElements(mxd, "TEXT_ELEMENT"):
                            if elm.name == "title":
                                elm.text = self.recipe.category + " map of " + self.event.country_name +\
                                    '\n' +\
                                    "<CLR red = '255'>Sheet - " + region + "</CLR>"
                            if elm.name == "map_no":
                                elm.text = self.recipe.mapnumber + "_Sheet_" + region.replace(' ', '_')

                        # Clear selection, otherwise the selected feature is highlighted in the exported map
                        arcpy.SelectLayerByAttribute_management(lyr, "CLEAR_SELECTION")
                        # Export to PDF
                        pdfFileName = coreFileName + "-" + \
                            slugify(unicode(region)) + "-" + str(self.event.default_pdf_res_dpi) + "dpi.pdf"
                        pdfFileLocation = os.path.join(exportDirectory, pdfFileName)

                        arcpy.mapping.ExportToPDF(mxd, pdfFileLocation, resolution=int(self.event.default_pdf_res_dpi))
                        if arcpy.Exists(os.path.join(exportDirectory, shpFile)):
                            arcpy.Delete_management(os.path.join(exportDirectory, shpFile))

        # TODO: asmith 2020/03/03
        # End of method nameded something like
        # _process_query_column_name(...)
        xmlExporter = XmlExporter(self.event, self.chef)
        exportParams['versionNumber'] = self.versionNumber
        exportParams['mapNumber'] = self.mapNumber
        exportParams['productName'] = self.productName
        exportParams['versionNumber'] = self.versionNumber
        exportParams["xmin"] = self.minx
        exportParams["ymin"] = self.miny
        exportParams["xmax"] = self.maxx
        exportParams["ymax"] = self.maxy
        exportXmlFileLocation = xmlExporter.write(exportParams)

        # TODO: asmith 2020/03/03
        # Seperate this section into a method named something like
        # _zip_exported_files(....)

        # And now Zip
        zipFileName = coreFileName+".zip"
        zipFileLocation = os.path.join(exportDirectory, zipFileName)

        with ZipFile(zipFileLocation, 'w') as zipObj:
            zipObj.write(exportXmlFileLocation, os.path.basename(exportXmlFileLocation))
            zipObj.write(jpgFileLocation, os.path.basename(jpgFileLocation))
            zipObj.write(pngThumbNailFileLocation, os.path.basename(pngThumbNailFileLocation))

            # TODO: asmith 2020/03/03
            # Given we are explictly setting the pdfFileName for each page within the DDPs
            # it is possible return a list of all of the filenames for all of the PDFs. Please
            # can we use this list to include in the zip file. There are edge cases where just
            # adding all of the pdfs in a particular directory might not behave correctly (eg if
            # the previous run had crashed midway for some reason)
            for pdf in os.listdir(exportDirectory):
                if pdf.endswith(".pdf"):
                    zipObj.write(os.path.join(exportDirectory, pdf),
                                 os.path.basename(os.path.join(exportDirectory, pdf)))
        print("Export complete to " + exportDirectory)

        # TODO: asmith 2020/03/03
        # End of method named something like
        # _zip_exported_files(....)

    def exportJpeg(self, coreFileName, exportDirectory, mxd, exportParams):
        # JPEG
        jpgFileName = coreFileName+"-"+str(self.event.default_jpeg_res_dpi) + "dpi.jpg"
        jpgFileLocation = os.path.join(exportDirectory, jpgFileName)
        exportParams["jpgFileName"] = jpgFileName
        arcpy.mapping.ExportToJPEG(mxd, jpgFileLocation)
        jpgFileSize = os.path.getsize(jpgFileLocation)
        exportParams["jpgFileSize"] = jpgFileSize
        return jpgFileLocation

    def exportPdf(self, coreFileName, exportDirectory, mxd, exportParams):
        # PDF
        pdfFileName = coreFileName+"-"+str(self.event.default_pdf_res_dpi) + "dpi.pdf"
        pdfFileLocation = os.path.join(exportDirectory, pdfFileName)
        exportParams["pdfFileName"] = pdfFileName
        arcpy.mapping.ExportToPDF(mxd, pdfFileLocation, resolution=int(self.event.default_pdf_res_dpi))
        pdfFileSize = os.path.getsize(pdfFileLocation)
        exportParams["pdfFileSize"] = pdfFileSize
        return pdfFileLocation

    def exportPngThumbNail(self, coreFileName, exportDirectory, mxd, exportParams):
        # PNG Thumbnail.  Need to create a larger image first.
        # If this isn't done, the thumbnail is pixelated amd doesn't look good
        pngTmpThumbNailFileName = "tmp-thumbnail.png"
        pngTmpThumbNailFileLocation = os.path.join(exportDirectory, pngTmpThumbNailFileName)
        arcpy.mapping.ExportToPNG(mxd, pngTmpThumbNailFileLocation)

        pngThumbNailFileName = "thumbnail.png"
        pngThumbNailFileLocation = os.path.join(exportDirectory, pngThumbNailFileName)

        # Resize the thumbnail
        fd_img = open(pngTmpThumbNailFileLocation, 'r+b')
        img = Image.open(fd_img)
        img = resizeimage.resize('thumbnail', img, [140, 99])
        img.save(pngThumbNailFileLocation, img.format)
        fd_img.close()

        # Remove the temporary larger thumbnail
        os.remove(pngTmpThumbNailFileLocation)
        return pngThumbNailFileLocation

    """
    Generates Export XML file

    Arguments:
        params {dict} -- Must contain the following parameters:
            * pdfFileName
            * pdfFileSize
            * jpgFileName
            * jpgFileSize
            * coreFileName
            * productType
            * exportDirectory

    Returns:
        Path to export XML file
    """


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
        return False
    else:
        return arg


def is_valid_directory(parser, arg):
    if os.path.isdir(arg):
        return arg
    else:
        parser.error("The directory %s does not exist!" % arg)
        return False


def add_bool_arg(parser, name, default=False):
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument('--' + name, dest=name, action='store_true')
    group.add_argument('--no-' + name, dest=name, action='store_false')
    parser.set_defaults(**{name: default})


# TODO: asmith 2020/03/03
# 1) Personally I am not convinced by the need to allow the user to manually specify or override
# all of the values that are typically included in the cmf_description.json file and/or the
# event_description.json (unless it is necessary for the ArcMap esriAddin).
#
# If this is desirable/required, then the wrangling and validation of those commandline args
# should be done outside of this class and in the mapactionpy_controller module.
#
# 2) I believe that there is a bug in the handling of the `--product` parameter. Also I take a
# different view as to what the default behaviour should be.
# BUG: At present the `--product` parameter is marked as optional, though I'm pretty sure that
# that a value of None would cause problems later on. I can't seen if a a string which ISN'T a
# product name in the mapCookbook.json is handled gracefully or not.
# Default behaviour: My prefered behaviour if no productName is specified, would be that that
# tool should attempt to create *all* of the products listed in the mapCookbook.json. Let's
# automate everything!
def main():
    parser = argparse.ArgumentParser(
        description='This component accepts a template MXD file, a list of the'
        'relevant datasets along with other information required to create an'
        'event specific instance of a map.',
    )
    parser.add_argument("-b", "--cookbook", dest="cookbookFile", required=False,
                        help="path to cookbook json file", metavar="FILE",
                        type=lambda x: is_valid_file(parser, x))
    parser.add_argument("-l", "--layerConfig", dest="layerConfig", required=False,
                        help="path to layer config json file", metavar="FILE",
                        type=lambda x: is_valid_file(parser, x))
    parser.add_argument("-t", "--template", dest="templateFile", required=False,
                        help="path to MXD file", metavar="FILE",
                        type=lambda x: is_valid_file(parser, x))
    parser.add_argument("-e", "--eventConfigFile", dest="eventConfigFile", required=True,
                        help="path the Event Config File", metavar="FILE",
                        type=lambda x: is_valid_file(parser, x))
    parser.add_argument("-ld", "--layerDirectory", dest="layerDirectory", required=False,
                        help="path to layer directory", metavar="FILE",
                        type=lambda x: is_valid_directory(parser, x))
    parser.add_argument("-p", "--product", dest="productName", required=True,
                        help="Name of product")
    parser.add_argument("-o", "--orientation", dest="orientation", default=None, required=False,
                        help="landscape|portrait")

    add_bool_arg(parser, 'export')

    args = parser.parse_args()
    orientation = None
    if (args.orientation is not None):
        if args.orientation.lower() == "landscape":
            orientation = args.orientation
        else:
            orientation = "portrait"

    eventConfig = Event(args.eventConfigFile, orientation)
    runner = ArcMapRunner(args.templateFile,
                          eventConfig,
                          args.productName)

    templateUpdated = runner.generate()
    if (templateUpdated):
        if (args.export):
            runner.export()
        else:
            print("Template updated.  No product export requested.")
    else:
        print("No product generated.  No changes since last execution.")


if __name__ == '__main__':
    main()



import argparse
import os
import arcpy

# TODO: asmith 2020/03/03
# What is the purpose of this file? Is this a tool that is just used within the
# maintainance of the `default_crash_move_folder`? It's entirely legitimate if it
# is, but it would be interesting to consider whether it could be intergrated into
# the other tools better. For example could we dump the layer files to text files on
# each commit of the `default_crash_move_folder` and keep the output within the git
# repo. This would mean to would be easier to see *what* had changed when a binary
# layer file changes.


def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
        return False
    else:
        return arg


def is_valid_directory(parser, arg):
    if os.path.isdir(arg):
        return arg
    else:
        parser.error("The directory %s does not exist!" % arg)
        return False


def get_layer_properties(file):
    lyr = arcpy.mapping.Layer(file)
    layerName = lyr.name.encode('utf-8').strip()

    if lyr.supports("DEFINITIONQUERY"):
        dq = lyr.definitionQuery.encode('utf-8').strip()
    else:
        dq = ""

    lblclassNames = []
    lblexpressions = []
    lblsql = []
    lblshow = False

    if lyr.supports("LABELCLASSES"):
        lblshow = lyr.showLabels
        for lblClass in lyr.labelClasses:
            if lblClass.showClassLabels:
                lblclassNames.append(lblClass.className.encode(
                    'utf-8').strip())
                lblexpressions.append(lblClass.expression.encode(
                    'utf-8').strip())
                lblsql.append(lblClass.SQLQuery.encode('utf-8').strip())

    return (layerName, dq, lblshow, lblclassNames, lblexpressions, lblsql)


def main(args):
    args = parser.parse_args()
    layerDirectory = args.layerDirectory

    # r=root, d=directories, f = files
    lyr_details = {}
    for r, d, f in os.walk(layerDirectory):
        for file in f:
            if '.lyr' in file:
                props = get_layer_properties(os.path.join(r, file))
                lyr_details[file] = props

    # Print one row per file:
    print("One row per file")
    print("fullPath|layerName|definitionQuery|lblshow|labelClassNameCount")
    for file in lyr_details:
        layerName, dq, lblshow, lblclassNames, lblexpressions, lblsql = lyr_details[file]
        print("|".join(map(str, (file, layerName, dq, lblshow, len(lblclassNames)))))

    print
    print
    print
    print("Only show files with Label Classes enabled and one row per *Label Class*")
    # Print one row per file:
    print("fullPath|layerName|definitionQuery|lblshow|labelClassNameCount|labelClassExpression|labelClassSQLQuery|")
    for file in lyr_details:
        layerName, dq, lblshow, lblclassNames, lblexpressions, lblsql = lyr_details[file]
        if len(lblclassNames):
            for n, e, s in zip(lblclassNames, lblexpressions, lblsql):
                print("|".join((file, layerName, dq, str(lblshow), n, e, s)))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        # TODO: asmith 2020/03/03
        # presumably "layer files" not "later files"?
        description='Dumps the contents of later files.',
    )
    parser.add_argument("-ld", "--layerDirectory", dest="layerDirectory", required=True,
                        help="path to layer directory", metavar="FILE", type=lambda x: is_valid_directory(parser, x))
    args = parser.parse_args()
    main(args)



import os
import arcpy
import jsonpickle
import re
from mapactionpy_controller.map_report import MapReport
from mapactionpy_controller.map_result import MapResult
from mapactionpy_controller.data_source import DataSource
from datetime import datetime

# TODO asmith 2020/03/06
# What is the separation of responsiblities between MapChef and ArcMapRunner? Why is the boundary
# between the two classes where it is? If I was to add a new function how would I know whether it
# should be added to MapChef or ArcMapRunner?
#
# Is it intended that the `cook()` method might be called multiple times in the life of a MapChef
# object? At present it looks to me like `cook()` can only be called once. In which case why have
# `cook()` as a public method and why not call it directly from the constructor.


class MapChef:
    """
    Worker which creates a Map based on a predefined "recipe" from a cookbook
    """
    # TODO asmith 2020/03/06
    # This constructor seem unnecessarily complicated. In the ArcMapRunner these objects are already
    # created:
    #   * MapCookbook object,
    #   * CrashMoveFolder object
    #   * LayerProperties object
    #   * Event object
    # It is already known that the various file and directory paths are valid etc. Why not just pass
    # those objects in as parameters to the MapChef constructor?
    #
    # Depending on whether or not it is indented that the `cook()` method might be called multiple
    # times in the life of a MapChef object, it would be worth reviewing

    def __init__(self,
                 mxd,
                 cookbook,
                 layerDefinition,
                 crashMoveFolder,
                 eventConfiguration,
                 versionNumber=1):
        """
        Arguments:
           mxd {MXD file} -- MXD file.
           cookbook {MapCookbook} -- Path to Map Cookbook json file
           layerPropertiesJsonFile {str} -- Path to Layer Properties json file
           crashMoveFolder {CrashMoveFolder} -- Path to Crash Move Folder json file
           eventConfiguration {Event} -- Event Object
           versionNumber {int} -- version number of map
        """
        # TODO asmith 2020/03/06
        # See comment on the `cook()` method about where and when the `mxd` parameter should be
        # passed.
        self.mxd = mxd
        self.crashMoveFolder = crashMoveFolder

        self.eventConfiguration = eventConfiguration
        self.cookbook = cookbook
        self.legendEntriesToRemove = list()

        self.datasetTypes = ["SHAPEFILE_WORKSPACE",
                             "RASTER_WORKSPACE",
                             "FILEGDB_WORKSPACE",
                             "ACCESS_WORKSPACE",
                             "ARCINFO_WORKSPACE",
                             "CAD_WORKSPACE",
                             "EXCEL_WORKSPACE",
                             "OLEDB_WORKSPACE",
                             "PCCOVERAGE_WORKSPACE",
                             "SDE_WORKSPACE",
                             "TEXT_WORKSPACE",
                             "TIN_WORKSPACE",
                             "VPF_WORKSPACE"]

        self.replaceDataSourceOnly = False
        # It appears that this is not used - therefore should be removed. If it is used, then it
        # TODO asmith 2020/03/06
        # needs a more specific name. There exist Data, Layerfile, MXD and Template Naming
        # Conventions (and possibly more)
        self.namingConvention = None
        # TODO asmith 2020/03/06
        # I suspect it would be more robust to set
        # self.summary = None
        self.summary = "Insert summary here"
        self.dataSources = set()
        self.versionNumber = versionNumber
        self.createDate = datetime.utcnow().strftime("%d-%b-%Y")
        self.createTime = datetime.utcnow().strftime("%H:%M")
        self.export = False

    def disableLayers(self):
        """
        Makes all layers invisible for all data-frames
        """
        for df in arcpy.mapping.ListDataFrames(self.mxd):
            for lyr in arcpy.mapping.ListLayers(self.mxd, "", df):
                lyr.visible = False

    def returnScale(self, dfscale):
        # https://community.esri.com/thread/163596
        scalebar = [2, 3, 4, 5, 6, 10]
        dfscale = dfscale/12
        dfscale = str(int(dfscale))
        dfscaleLen = len(dfscale)
        numcheck = int(dfscale[0])
        for each in scalebar:
            if numcheck < each:
                multi = '1'
                while dfscaleLen > 1:
                    multi = multi + '0'
                    dfscaleLen = dfscaleLen - 1
                scalebar = each * int(multi)
                dataframescale = scalebar * 12
                return scalebar, dataframescale

    def scale(self):
        newScale = ""
        for df in arcpy.mapping.ListDataFrames(self.mxd):
            if df.name.lower() == "main map":
                intValue = '{:,}'.format(int(df.scale))
                newScale = "1: " + intValue + " (At A3)"
                break
        return newScale

    def spatialReference(self):
        spatialReferenceString = "Unknown"
        for df in arcpy.mapping.ListDataFrames(self.mxd):
            if df.name.lower() == "main map":
                if (len(df.spatialReference.datumName) > 0):
                    spatialReferenceString = df.spatialReference.datumName
                    spatialReferenceString = spatialReferenceString[2:]
                    spatialReferenceString = spatialReferenceString.replace('_', ' ')
                break
        return spatialReferenceString

    # TODO asmith 2020/0306
    # Do we need to accommodate a use case where we would want to add layers but not make them
    # visible? If so is this something that we deal with when we get to it?
    def enableLayers(self):
        """
        Makes all layers visible for all data-frames
        """
        for df in arcpy.mapping.ListDataFrames(self.mxd):
            for lyr in arcpy.mapping.ListLayers(self.mxd, "", df):
                lyr.visible = True

    def removeLayers(self):
        """
        Removes all layers for all data-frames
        """
        for df in arcpy.mapping.ListDataFrames(self.mxd):
            for lyr in arcpy.mapping.ListLayers(self.mxd, "", df):
                if (lyr.longName != "Data Driven Pages"):
                    arcpy.mapping.RemoveLayer(df, lyr)
        self.mxd.save()

    # TODO asmith 2020/03/06
    # I would suggest that:
    #   * If `cook() only gets called once in the life of a MapChef object, then it should be
    #     entire procedural, with no parameters (everything set via the constructor) and
    #     subsequent attempt to call `cook()` should result in an exception
    #   * If `cook()` can be called multiple times, then the `mxd` and the `map_version_number`
    #     should be parameters for the cook method and not for the constructor.
    #
    # Not with standing the above, the relevant Recipe object has already be identified and
    # validated in the ArcMapRunner object. Why not just pass that instead of the productName?

    def cook(self, recipe, replaceDataSourceOnly=False):
        self.recipe = recipe
        self.replaceDataSourceOnly = replaceDataSourceOnly
        arcpy.env.addOutputsToMap = False
        if not replaceDataSourceOnly:
            self.disableLayers()
            self.removeLayers()

        self.mapReport = MapReport(self.recipe.product)
        if (self.recipe is not None):
            if len(self.recipe.summary):
                self.summary = self.recipe.summary
            for mf in self.recipe.map_frames:
                for layer in mf.layers:
                    self.process_layer(layer, mf)

        self.enableLayers()
        arcpy.RefreshTOC()
        arcpy.RefreshActiveView()
        arcpy.env.addOutputsToMap = True
        self.showLegendEntries()
        self.mxd.save()

        if (recipe is not None):
            self.updateTextElements()
            self.mxd.save()

    """
    Adds data file to map layer

    Can handle the following file types:
        * Shapefiles
        * IMG files
        * TIF files

    Arguments:
        dataFrame {str} -- Name of data frame to add data source file to
        dataFile {str}  -- Full path to data file
        layer {arcpy._mapping.Layer} -- Layer to which data is added
        definitionQuery {str} -- Some layers have a definition query which select specific features from a SQL query
        labelClasses {list} -- List of LabelClass objects

    Returns:
        boolean -- added (true if successful)
    """

    def addDataToLayer(self,
                       dataFrame,
                       dataFile,
                       layer,
                       definitionQuery,
                       datasetName,
                       labelClasses,
                       addToLegend,
                       zoomMultiplier=0):
        added = False
        for lyr in arcpy.mapping.ListLayers(layer):
            if lyr.supports("LABELCLASSES"):
                for labelClass in labelClasses:
                    for lblClass in lyr.label_classes:
                        if (lblClass.className == labelClass.className):
                            lblClass.SQLQuery = labelClass.sql_query.replace('{COUNTRY_NAME}',
                                                                             self.eventConfiguration.country_name)
                            lblClass.expression = labelClass.expression
                            lblClass.showClassLabels = labelClass.show_class_labels
            if lyr.supports("DATASOURCE"):  # An annotation layer does not support DATASOURCE
                for datasetType in self.datasetTypes:
                    try:
                        lyr.replaceDataSource(dataFile, datasetType, datasetName)
                        added = True
                    except Exception:
                        pass

                    if ((added is True) and (definitionQuery)):
                        definitionQuery = definitionQuery.replace('{COUNTRY_NAME}',
                                                                  self.eventConfiguration.country_name)
                        lyr.definition_query = definitionQuery
                        try:
                            arcpy.SelectLayerByAttribute_management(lyr, "SUBSET_SELECTION", definitionQuery)
                        except Exception:
                            added = False

                    if (added is True):
                        if addToLegend is False:
                            self.legendEntriesToRemove.append(lyr.name)
                            if (self.namingConvention is not None):
                                dnr = self.namingConvention.validate(datasetName)
                                # We want to capture Description:
                                if 'Description' in dnr.source._fields:
                                    if (dnr.source.Description.lower() not in ('unknown', 'undefined', 'mapaction')):
                                        self.dataSources.add(dnr.source.Description)

                        if (self.replaceDataSourceOnly):
                            self.mxd.save()
                        else:
                            arcpy.mapping.AddLayer(dataFrame, lyr, "BOTTOM")
                        break
                lyr.visible = False
                self.applyZoom(dataFrame, lyr, zoomMultiplier)

        return added

    def report_as_json(self):
        """
        Returns map report in json format
        """
        return(jsonpickle.encode(self.mapReport, unpicklable=False))

    def process_layer(self, recipe_lyr, map_frame):
        """
        Updates or Adds a layer of data.  Maintains the Map Report.
        """
        mapResult = MapResult(recipe_lyr.name)
        self.dataFrame = arcpy.mapping.ListDataFrames(self.mxd, map_frame.name)[0]
        try:
            arc_lyr_to_update = arcpy.mapping.ListLayers(self.mxd, recipe_lyr.name, self.dataFrame)[0]
            # Replace existing layer
            mapResult = self.updateLayer(arc_lyr_to_update, recipe_lyr)
        except IndexError:
            # Layer doesn't exist, add new layer
            mapResult = self.addLayer(recipe_lyr)

        if mapResult.added:
            try:
                # Seperate the next two lines so that the cause of any exceptions is more easily
                # appartent from the stack trace.
                lyr_list = arcpy.mapping.ListLayers(self.mxd, recipe_lyr.name, self.dataFrame)
                newLayer = lyr_list[0]
                self.applyZoom(self.dataFrame, newLayer, recipe_lyr.get('zoomMultiplier', 0))
            except IndexError:
                pass

        self.mapReport.add(mapResult)

    def find(self, rootdir, regexp, gdb=False):
        returnPaths = list()
        # TODO asmith 2020/03/06
        # What is the purpose of wrangling the regexps here?
        # The regexs in the layerProperties.json just match the filenames. The purpose of this
        # seems to be to change the regexs to work on the full path, and then later join the full
        # filename with the directory path before attempting to match the regexs.
        # I suspect there are some edge cases where incorrectly named files could inadvertantly
        # matched here.
        regexp = regexp.replace("^", "\\\\")
        regexp = regexp.replace("/", "\\\\")
        regexp = ".*" + regexp
        re.compile(regexp)
        for root, dirs, files in os.walk(os.path.abspath(rootdir)):
            if (gdb is False):
                for file in files:
                    filePath = os.path.join(root, file)
                    z = re.match(regexp, filePath)
                    if (z):
                        # TODO asmith 2020/03/06
                        # Is this necessary? Having a `$` at the end of the regex would have the
                        # effect of as excluding the lock files.
                        if not(filePath.endswith("lock")):
                            returnPaths.append(filePath)
            else:
                for dir in dirs:
                    dirPath = os.path.join(root, dir)
                    z = re.match(regexp, dirPath)
                    if (z):
                        returnPaths.append(dirPath)
        return returnPaths

    """
    Updates Text Elements in Marginalia

    """

    def updateTextElements(self):
        for elm in arcpy.mapping.ListLayoutElements(self.mxd, "TEXT_ELEMENT"):
            if elm.name == "country":
                elm.text = self.eventConfiguration.country_name
            if elm.name == "title":
                elm.text = self.recipe.product
            if elm.name == "create_date_time":
                elm.text = self.createDate + " " + self.createTime
            if elm.name == "summary":
                elm.text = self.summary
            if elm.name == "map_no":
                elm.text = self.recipe.mapnumber
            if elm.name == "mxd_name":
                elm.text = os.path.basename(self.mxd.filePath)
            if elm.name == "scale":
                elm.text = self.scale()
            if elm.name == "data_sources":
                iter = 0
                dataSourcesString = "<BOL>Data Sources:</BOL>" + os.linesep + os.linesep
                for ds in self.dataSources:
                    if (iter > 0):
                        dataSourcesString = dataSourcesString + ", "
                    dataSourcesString = dataSourcesString + ds
                    iter = iter + 1
                elm.text = dataSourcesString
            if elm.name == "map_version":
                versionNumberString = "v" + str(self.versionNumber).zfill(2)
                elm.text = versionNumberString
            if elm.name == "spatial_reference":
                elm.text = self.spatialReference()
            if elm.name == "glide_no":
                if (self.eventConfiguration is not None):
                    elm.text = self.eventConfiguration.glide_number
            if elm.name == "donor_credit":
                if (self.eventConfiguration is not None):
                    elm.text = self.eventConfiguration.default_donor_credits
            if elm.name == "disclaimer":
                if (self.eventConfiguration is not None):
                    elm.text = self.eventConfiguration.default_disclaimer_text
            if elm.name == "map_producer":
                if (self.eventConfiguration is not None):
                    elm.text = "Produced by " + \
                        self.eventConfiguration.default_source_organisation + \
                        os.linesep + \
                        self.eventConfiguration.deployment_primary_email + \
                        os.linesep + \
                        self.eventConfiguration.default_source_organisation_url
        self.mxd.save()

    def showLegendEntries(self):
        for legend in arcpy.mapping.ListLayoutElements(self.mxd, "LEGEND_ELEMENT"):
            layerNames = list()
            for lyr in legend.listLegendItemLayers():
                if ((lyr.name in self.legendEntriesToRemove) or (lyr.name in layerNames)):
                    legend.removeItem(lyr)
                else:
                    layerNames.append(lyr.name)
        self.mxd.save()

    # TODO asmith 2020/03/06
    # Please don't hard code size and location of elements on the template
    def alignLegend(self, orientation):
        for legend in arcpy.mapping.ListLayoutElements(self.mxd, "LEGEND_ELEMENT"):
            if orientation == "landscape":
                # Resize
                legend.elementWidth = 60
                legend.elementPositionX = 248.9111
                legend.elementPositionY = 40
        self.mxd.save()

    # TODO asmith 2020/03/06
    # Please don't hard code size and location of elements on the template
    def resizeScaleBar(self):
        elm = arcpy.mapping.ListLayoutElements(self.mxd, "MAPSURROUND_ELEMENT", "Scale Bar")[0]
        elm.elementWidth = 51.1585
        self.mxd.save()

    def applyZoom(self, dataFrame, lyr, zoomMultiplier):
        if (zoomMultiplier != 0):
            buffer = zoomMultiplier
            arcpy.env.overwriteOutput = "True"
            extent = lyr.getExtent(True)  # visible extent of layer

            extBuffDist = ((int(abs(extent.lowerLeft.X - extent.lowerRight.X))) * buffer)

            # TODO asmith 2020/03/06
            # This is untested but possibly much terser:
            # ```
            #        x_min = extent.XMin - extBuffDist
            #        y_min = extent.YMin - extBuffDist
            #        x_max = extent.XMax + extBuffDist
            #        y_max = extent.YMax + extBuffDist
            #        new_extent = arcpy.Extent(x_min, y_min, x_max, y_max)
            #        dataFrame.extent = new_extent
            # ```

            newExtentPts = arcpy.Array()
            newExtentPts.add(arcpy.Point(extent.lowerLeft.X-extBuffDist,
                                         extent.lowerLeft.Y-extBuffDist,
                                         extent.lowerLeft.Z,
                                         extent.lowerLeft.M,
                                         extent.lowerLeft.ID))

            newExtentPts.add(arcpy.Point(extent.lowerRight.X+extBuffDist,
                                         extent.lowerRight.Y-extBuffDist,
                                         extent.lowerRight.Z,
                                         extent.lowerRight.M,
                                         extent.lowerRight.ID))

            newExtentPts.add(arcpy.Point(extent.upperRight.X+extBuffDist,
                                         extent.upperRight.Y+extBuffDist,
                                         extent.upperRight.Z,
                                         extent.upperRight.M,
                                         extent.upperRight.ID))

            newExtentPts.add(arcpy.Point(extent.upperLeft.X-extBuffDist,
                                         extent.upperLeft.Y+extBuffDist,
                                         extent.upperLeft.Z,
                                         extent.upperLeft.M,
                                         extent.upperLeft.ID))

            newExtentPts.add(arcpy.Point(extent.lowerLeft.X-extBuffDist,
                                         extent.lowerLeft.Y-extBuffDist,
                                         extent.lowerLeft.Z,
                                         extent.lowerLeft.M,
                                         extent.lowerLeft.ID))
            polygonTmp2 = arcpy.Polygon(newExtentPts)
            dataFrame.extent = polygonTmp2
            self.mxd.save()

    # TODO: asmith 2020/03/06
    # `updateLayer()` and `addLayer()` seem very similar. Is it possible to refactor to reduce
    # duplication?
    def updateLayer(self, arc_lyr_to_update, recipe_lyr):
        mapResult = None

        if (".gdb/" not in recipe_lyr.reg_exp):
            mapResult = self.updateLayerWithFile(recipe_lyr, arc_lyr_to_update, recipe_lyr.layer_file_path)
        else:
            mapResult = self.updateLayerWithGdb(recipe_lyr)
        return mapResult

    # TODO: asmith 2020/03/06
    # `updateLayer()` and `addLayer()` seem very similar. Is it possible to refactor to reduce
    # duplication?
    def addLayer(self, recipe_lyr):
        # addLayer(recipe_lyr, recipe_lyr.layer_file_path, recipe_lyr.name)
        mapResult = MapResult(recipe_lyr.name)
        arc_lyr_to_add = arcpy.mapping.Layer(recipe_lyr.layer_file_path)
        if (".gdb/" not in recipe_lyr.reg_exp):
            mapResult = self.addLayerWithFile(recipe_lyr, arc_lyr_to_add, recipe_lyr.name)
        else:
            mapResult = self.addLayerWithGdb(recipe_lyr, arc_lyr_to_add, recipe_lyr.name)
        return mapResult

    # TODO: asmith 2020/03/06
    # These three methods appear very similar:
    #   * `addLayerWithFile()`
    #   * `addLayerWithGdb()`
    #   * `updateLayerWithFile()`
    # Is it possible to refactor to reduce duplication?
    def updateLayerWithFile(self, layerProperties, updateLayer, layerFilePath):
        mapResult = MapResult(layerProperties.name)

        dataFiles = self.find(self.crashMoveFolder.active_data, layerProperties.reg_exp)
        for dataFile in (dataFiles):
            base = os.path.basename(dataFile)
            datasetName = os.path.splitext(base)[0]
            dataDirectory = os.path.dirname(os.path.realpath(dataFile))

            sourceLayer = arcpy.mapping.Layer(layerFilePath)
            arcpy.mapping.UpdateLayer(self.dataFrame, updateLayer, sourceLayer, False)

            newLayer = arcpy.mapping.ListLayers(self.mxd, updateLayer.name, self.dataFrame)[0]
            if newLayer.supports("DATASOURCE"):
                for datasetType in self.datasetTypes:
                    try:
                        if (newLayer.supports("DEFINITIONQUERY") and (layerProperties.definition_query)):
                            newLayer.definition_query = layerProperties.definition_query.replace(
                                '{COUNTRY_NAME}', self.eventConfiguration.country_name)
                        newLayer.replaceDataSource(dataDirectory, datasetType, datasetName)
                        mapResult.message = "Layer updated successfully"
                        mapResult.added = True
                        ds = DataSource(dataFile)
                        mapResult.dataSource = dataFile.replace("\\", "/").replace(self.crashMoveFolder.path.replace("\\", "/"), "")   # noqa
                        mapResult.hash = ds.calculate_checksum()
                        break
                    except Exception:
                        pass

            if (mapResult.added is True):
                self.mxd.save()
                break
        return mapResult

    def updateLayerWithGdb(self, layerProperties):
        mapResult = MapResult(layerProperties.name)
        mapResult.message = "Update layer for a GeoDatabase not yet implemented"
        return mapResult

    # TODO: asmith 2020/03/06
    # These three methods appear very similar:
    #   * `addLayerWithFile()`
    #   * `addLayerWithGdb()`
    #   * `updateLayerWithFile()`
    # Is it possible to refactor to reduce duplication?
    def addLayerWithFile(self, layerProperties, layerToAdd, cookBookLayer):
        mapResult = MapResult(layerProperties.name)
        dataFiles = self.find(self.crashMoveFolder.active_data, layerProperties.reg_exp)

        for dataFile in (dataFiles):
            base = os.path.basename(dataFile)
            datasetName = os.path.splitext(base)[0]
            dataDirectory = os.path.dirname(os.path.realpath(dataFile))

            if layerToAdd.supports("LABELCLASSES"):
                for labelClass in layerProperties.label_classes:
                    for lblClass in layerToAdd.labelClasses:
                        if (lblClass.className == labelClass.class_name):
                            lblClass.SQLQuery = labelClass.sql_query.replace('{COUNTRY_NAME}',
                                                                             self.eventConfiguration.country_name)
                            lblClass.expression = labelClass.expression
                            lblClass.showClassLabels = labelClass.show_class_labels

            if layerToAdd.supports("DATASOURCE"):
                for datasetType in self.datasetTypes:
                    try:
                        layerToAdd.replaceDataSource(dataDirectory, datasetType, datasetName)
                        mapResult.message = "Layer added successfully"
                        mapResult.added = True
                        ds = DataSource(dataFile)
                        mapResult.dataSource = dataFile.replace("\\", "/").replace(self.crashMoveFolder.path.replace("\\", "/"), "")   # noqa
                        mapResult.hash = ds.calculate_checksum()
                        break
                    except Exception:
                        pass

            if ((mapResult.added is True) and (layerProperties.definition_query)):
                definitionQuery = layerProperties.definition_query.replace('{COUNTRY_NAME}',
                                                                           self.eventConfiguration.country_name)
                layerToAdd.definition_query = definitionQuery
                try:
                    arcpy.SelectLayerByAttribute_management(layerToAdd,
                                                            "SUBSET_SELECTION",
                                                            layerProperties.definition_query)
                except Exception:
                    mapResult.added = False
                    mapResult.message = "Selection query failed: " + layerProperties.definition_query
                    self.mxd.save()

            if (mapResult.added is True):
                # TODO add proper fix for applyZoom in line with these two cards
                # https: // trello.com/c/Bs70ru1s/145-design-criteria-for-selecting-zoom-extent
                # https://trello.com/c/piE3tKRp/146-implenment-rules-for-selection-zoom-extent
                # self.applyZoom(self.dataFrame, layerToAdd, cookBookLayer.get('zoomMultiplier', 0))
                self.applyZoom(self.dataFrame, layerToAdd, 0)

                if layerProperties.add_to_legend is False:
                    self.legendEntriesToRemove.append(layerToAdd.name)
                arcpy.mapping.AddLayer(self.dataFrame, layerToAdd, "BOTTOM")
                self.mxd.save()
                break

        return mapResult

    # TODO: asmith 2020/03/06
    # These three methods appear very similar:
    #   * `addLayerWithFile()`
    #   * `addLayerWithGdb()`
    #   * `updateLayerWithFile()`
    # Is it possible to refactor to reduce duplication?
    def addLayerWithGdb(self, layerProperties, layerToAdd, cookBookLayer):
        mapResult = MapResult(layerProperties.name)

        # It's a File Geodatabase
        parts = layerProperties.reg_exp.split("/")
        gdbPath = parts[0]
        geoDatabases = self.find(self.crashMoveFolder.active_data, gdbPath, True)
        for geoDatabase in geoDatabases:
            arcpy.env.workspace = geoDatabase
            rasters = arcpy.ListRasters("*")
            for raster in rasters:
                if re.match(parts[1], raster):
                    self.dataFrame = arcpy.mapping.ListDataFrames(self.mxd, layerProperties.mapFrame)[0]
                    mapResult.added = self.addDataToLayer(self.dataFrame,
                                                          geoDatabase,
                                                          layerToAdd,
                                                          layerProperties.definition_query,
                                                          raster,
                                                          layerProperties.label_classes,
                                                          layerProperties.add_to_legend)

                    dataFile = geoDatabase + os.sep + raster
                    ds = DataSource(dataFile)
                    mapResult.dataSource = dataFile.replace("\\", "/").replace(self.crashMoveFolder.path.replace("\\", "/"), "")  # noqa
                    mapResult.hash = ds.calculate_checksum()
                    break
            featureClasses = arcpy.ListFeatureClasses()
            for featureClass in featureClasses:
                if re.match(parts[1], featureClass):
                    # Found Geodatabase.  Stop iterating.
                    self.dataFrame = arcpy.mapping.ListDataFrames(self.mxd, layerProperties.mapFrame)[0]
                    mapResult.added = self.addDataToLayer(self.dataFrame,
                                                          geoDatabase,
                                                          layerToAdd,
                                                          layerProperties.definition_query,
                                                          featureClass,
                                                          layerProperties.label_classes,
                                                          layerProperties.add_to_legend)
                    dataFile = geoDatabase + os.sep + featureClass
                    ds = DataSource(dataFile)
                    mapResult.dataSource = dataFile.replace("\\", "/").replace(self.crashMoveFolder.path.replace("\\", "/"), "")  # noqa
                    mapResult.hash = ds.calculate_checksum()
                    break

        return mapResult



import arcpy
import os
# import unittest
from unittest import TestCase
from mapactionpy_arcmap.map_chef import MapChef
from mapactionpy_controller.crash_move_folder import CrashMoveFolder
from mapactionpy_controller.map_cookbook import MapCookbook
from mapactionpy_controller.event import Event
from mapactionpy_controller.layer_properties import LayerProperties


class TestMapChef(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        self.path_to_valid_cmf_des = os.path.join(
            self.parent_dir, 'tests', 'test_data', 'fixture_cmf_description_flat_test.json')
        self.cmf = CrashMoveFolder(self.path_to_valid_cmf_des)
        self.event = Event(os.path.join(self.parent_dir, 'tests', 'test_data',
                                        'event_description.json'))
        self.my_mxd_fname = os.path.join(self.parent_dir, 'tests', 'test_data',
                                         'output_arcgis_10_6_reference_landscape_bottom.mxd')
        self.layer_props = LayerProperties(self.cmf, '.lyr')
        self.cookBook = MapCookbook(self.cmf, self.layer_props)

    def test_map_chef_constructor(self):
        my_mxd = arcpy.mapping.MapDocument(self.my_mxd_fname)

        mc = MapChef(
            my_mxd,
            self.cookBook,
            self.layer_props,
            self.cmf,
            self.event,
            versionNumber=1
        )
        self.assertIsInstance(mc, MapChef)

    def test_map_chef_cook(self):
        my_mxd = arcpy.mapping.MapDocument(self.my_mxd_fname)
        productName = "Example Map"

        mc = MapChef(
            my_mxd,
            self.cookBook,
            self.layer_props,
            self.cmf,
            self.event,
            versionNumber=1
        )

        mc.cook(self.cookBook.products[productName])
        self.assertTrue(True)



import os
import sys
# import unittest
from unittest import TestCase

from mapactionpy_controller.crash_move_folder import CrashMoveFolder
import mapactionpy_arcmap.arcmap_runner as arcmap_runner


class TestArcMapRunner(TestCase):

    def setUp(self):
        self.parent_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        self.path_to_valid_cmf_des = os.path.join(
            self.parent_dir, 'tests', 'test_data', 'fixture_cmf_description_flat_test.json')
        self.cmf = CrashMoveFolder(self.path_to_valid_cmf_des)

    def test_arcmap_runner_main(self):
        sys.argv[1:] = ['--eventConfigFile', os.path.join(self.cmf.path, 'event_description.json'),
                        '--template', os.path.join(self.cmf.map_templates,
                                                   'arcgis_10_6_reference_landscape_bottom.mxd'),
                        "--product", "Example Map"]

        arcmap_runner.main()
        self.assertTrue(True)

    def test_arcmap_runner_main_unknown_product(self):
        sys.argv[1:] = ['--eventConfigFile', os.path.join(self.cmf.path, 'event_description.json'),
                        '--template', os.path.join(self.cmf.map_templates,
                                                   'arcgis_10_6_reference_landscape_bottom.mxd'),
                        "--product", "This product does not exist"]
        try:
            arcmap_runner.main()
        except Exception as e:
            self.assertTrue("Could not find recipe for product: \"" +
                            sys.argv[6] + "\"" in str(e.message))



# flake8: noqa
# from .label_class import LabelClass
# from .map_layer import MapLayer
# from .layer_properties import LayerProperties
# from .map_recipe import MapRecipe
from .map_chef import MapChef
# from .map_cookbook import MapCookbook
# from .map_report import MapReport
# from .map_result import MapResult



import subprocess
from setuptools import setup, find_packages
from os import path, environ

_base_version = '0.12.2'
root_dir = path.abspath(path.dirname(__file__))


def readme():
    with open(path.join(root_dir, 'README.md')) as f:
        return f.read()


def _get_version_number():
    travis_build = environ.get('TRAVIS_BUILD_NUMBER')
    travis_tag = environ.get('TRAVIS_TAG')

    if travis_build:
        if travis_tag:
            version = travis_tag
        else:
            version = '{}.dev{}'.format(_base_version, travis_build)

        with open(path.join(root_dir, 'VERSION'), 'w') as version_file:
            version_file.write(version.strip())
    else:
        try:
            ver = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'])
            version = '{}+local.{}'.format(_base_version, ver.decode('ascii').strip())
        except Exception:
            with open(path.join(root_dir, 'VERSION')) as version_file:
                version = version_file.read().strip()

    return version


# TODO: asmith
# install_requires should include `mapactionpy_controller`
setup(name='mapactionpy_arcmap',
      version=_get_version_number(),
      description='Used to drive ArcMap',
      long_description=readme(),
      long_description_content_type="text/markdown",
      url='http://github.com/mapaction/mapactionpy_arcmap',
      author='MapAction',
      author_email='github@mapaction.com',
      license='GPL3',
      packages=find_packages(),
      install_requires=[
          'mapactionpy_controller',
          'argparse',
          'slugify',
          'jsonpickle',
          'Pillow',
          'python-resize-image'
      ],
      test_suite='unittest',
      tests_require=['unittest'],
      zip_safe=False,
      classifiers=[
          "Development Status :: 2 - Pre-Alpha",
          "Programming Language :: Python :: 2.7",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
          "Operating System :: Microsoft :: Windows",
      ])



import os

this_dir = os.path.abspath(os.path.dirname(__file__))
example_cookbook_path = os.path.join(this_dir, 'Config', 'mapCookbook.json')
example_lyr_props__path = os.path.join(this_dir, 'Config', 'layerProperties.json')

output_cookbook_path = os.path.join(this_dir, 'Config', 'mapCookbook.v2.json')
output_lyr_props__path = os.path.join(
    this_dir, 'Config', 'layerProperties.v2.json')


find_and_replace = [
    ('Elevation - Hillshade', 'mainmap-s0-ras-hillshade'),
    ('Transport - Airports', 'mainmap-s0-pt-airports'),
    ('Transport - Seaports', 'mainmap-s0-pt-seaports'),
    ('Admin 0 - Affected Country', 'mainmap-s0-py-affectedcountry'),
    ('Transport - Rail', 'mainmap-s0-ln-rail'),
    ('Physical - Sea', 'mainmap-s0-py-sea'),
    ('Admin - Ad 1 Polygon', 'mainmap-s0-py-admin1'),
    ('Admin - Ad 2 Polygon', 'mainmap-s0-py-admin2'),
    ('Elevation - Coastline', 'mainmap-s1-ln-coastline'),
    ('Transport - Roads', 'mainmap-s0-ln-roads'),
    ('Elevation - Curvature', 'mainmap-s0-ras-curvature'),
    ('Elevation - DEM', 'mainmap-s0-ras-dem'),
    ('Admin - Ad 4 Polygon', 'mainmap-s3-py-admin4'),
    ('Admin 0 - Surrounding Country', 'mainmap-s0-py-surroundingcountries'),
    ('Physical - Lakes', 'mainmap-s0-py-waterbodies'),
    ('Cartography - Feather', 'mainmap-s0-py-feather'),
    ('Borders - Admin 1', 'mainmap-s1-ln-admin1'),
    ('Borders - Admin 2', 'mainmap-s1-ln-admin2'),
    ('Admin - Ad 3 Polygon', 'mainmap-s2-py-admin3'),
    ('Settlements - Places', 'mainmap-s0-pt-settlements'),
    ('Physical - Rivers', 'mainmap-s0-ln-rivers'),
    ('Legend - Roads', 'legend-s0-ln-road'),
    ('Provinces', 'mainmap-s0-py-admin1'),
    ('Location - Surrounding Country', 'locationmap-s0-py-surroundingcountries'),
    ('Legend - Water', 'legend-s0-py-waterbodies'),
    ('Elevation - DEM EXCLUDE', 'mainmap-s0-ras-dem'),
    ('Borders - Admin 4', 'mainmap-s3-py-admin4'),
    ('Physical - Waterbodies', 'mainmap-s0-py-waterbodies'),
    ('Borders - Admin 3', 'mainmap-s2-ln-admin3'),
    ('229_stle', '228_stle'),
    ('229_stle', '228_stle'),
    ('232_tran', '230_tran'),
    ('221_phys', '220_phys'),
    ('223_popu', '222_popu')
]


def update_file(infilepath, outfilepath):
    with open(infilepath, 'r') as infile:
        s = infile.read()

    for find, replace in find_and_replace:
        s = s.replace(find, replace)

    with open(outfilepath, 'w') as outfile:
        outfile.write(s)


if __name__ == "__main__":
    update_file(example_cookbook_path, output_cookbook_path)
    update_file(example_lyr_props__path, output_lyr_props__path)
